<?php

	require_once("config.php");

	// This file contains functions for gathering an authors details
	// from Google+ and Twitter. It uses APIs where possible, and does
	// a bit of scraping to fill in gaps as necessary.

	// Connects to the Google+ API to collect details
	// about the author.
	function get_author_details_via_api($author_id, $depth = 1)
	{
		global $gplus_key;
		
		set_time_limit(120);
	
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, "https://www.googleapis.com/plus/v1/people/" . $author_id . "?key=" . $gplus_key);
		curl_setopt($ch, CURLOPT_TIMEOUT, 180);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		
		$person_json = curl_exec($ch);
		$info = curl_getinfo($ch);
		curl_close($ch);
		
		// If the API failed (rate limit), then pause for 5 seconds
		// before trying again. However, the depth parameter prevents
		// us sitting here forever.
		if (($info['http_code'] != "200") && ($depth <= 3))
		{
			echo "Waiting on Google+ API, following a problem...";
			sleep(5);
			return get_author_details_via_api($author_id, $depth+1);
		}
	    $author_details = json_decode($person_json,true);
	    
		return $author_details;
	}
	
	// This function fills in gaps not available via the API
	// Primarily, this is number of circles they are in, and
	// identifying which URLs they are contributor to.
	function get_author_details_via_scrape($author_id)
	{
		$author_details = array();
		
		// Fetch the profile page for the user.
		set_time_limit(120);
	
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, "https://plus.google.com/" . $author_id . "/about");
		curl_setopt($ch, CURLOPT_TIMEOUT, 180);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		
		$profile_page = curl_exec($ch);
		//$info = curl_getinfo($ch);
		curl_close($ch);
		
		// Scrape out the number of circles the user is in.
		//$circles_pattern = '/(<h4 class=\"nPQ0Mb pD8zNd\">Have)(.*in circles \()([0-9]+)(.*)/'; // This was before the April 11th 2012 update that changed the layout
		$circles_pattern = "/(\{key: \'77\')(.*)(\[)([0-9]+)(,\[\[\\\\\")(.*)/"; // This is the post April 11th version
		
		$author_details['in_circles'] = 'unknown';
		
		if (preg_match($circles_pattern, $profile_page, $matches))
			$author_details['in_circles'] = intval($matches[4]);
	
		// Now scrape the contributes to URLs.
		
		// Not a terribly robust regex here. Will break if they futz with profile page HTML
		$contributor_to_pattern = '/(<a class=[\'\"]\w*\s*url[\'\"]\s*href=[\'\"])([^\'^\"]*)([\'\"]\s*rel=[\'\"]contributor-to[^>]*)>/';
		preg_match_all($contributor_to_pattern, $profile_page, $contributor_matches);
		
		$author_details['contributes_to_urls'] = array();
		
		foreach($contributor_matches[2] as $contributes_to_url)
		{
			$author_details['contributes_to_urls'][] = $contributes_to_url;
		}
		
		return $author_details;
	}

	// Checks the Google+ profile URLs listed for a Twitter handle
	// then if one is found it also fetches the follower count.
	function get_author_twitter_details($author_details_from_google_plus_api)
	{
		$twitter_details = array();
		//https://api.twitter.com/1/users/show.json?screen_name=TomAnthonySEO
		
		$circles_pattern = '/(.*)(twitter.com\/)(\!\#\/)([^&]*)(.*)/';
		preg_match($circles_pattern, $profile_page, $matches);
		
		$twitter_details['twitter_screename'] = 'unknown';
		$twitter_details['twitter_follower_count'] = 'unknown';
		
		foreach($author_details_from_google_plus_api['urls'] as $candidate_url)
		{
			$profile_name_pattern = '/(.*)(twitter\.com\/)([\!\#\/]*)([^\?]+)(.*)/';
			
			if (preg_match($profile_name_pattern, $candidate_url['value'], $matches))
			{
				$twitter_details['twitter_screename'] = $matches[4];
				$twitter_details['twitter_follower_count'] = get_twitter_follower_count($twitter_details['twitter_screename']);
				
				return $twitter_details;
			}
		}
		
		return $twitter_details;
	}
	
	// Connects to the Twitter API to get follower count.
	function get_twitter_follower_count($twitter_screename)
	{
		set_time_limit(120);
	
		$ch = curl_init();
		
		curl_setopt($ch, CURLOPT_URL, "https://api.twitter.com/1/users/show.json?screen_name=" . $twitter_screename);
		curl_setopt($ch, CURLOPT_TIMEOUT, 180);
		curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, false);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		
		$twitter_details_json = curl_exec($ch);
		curl_close($ch);
		
	    $twitter_details = json_decode($twitter_details_json,true);
		
		return $twitter_details['followers_count'];
	}
?>